﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SqlServer.SSIS.EzAPI;
using Microsoft.SqlServer.Dts.Design;
using Microsoft.SqlServer.Dts.Runtime;
using Dimodelo.MetaData;
using System.Xml.Linq;
using System.Xml.XPath;

namespace com.Dimodelo.ezAPIExamples
{

    // DEMO1: This package contains a single Execute package task

    public class HelloWorldEzAPIExampleWithDimodelo : EzPackage, ezAPIGenerationTemplate
    {

        // Provide this constructor only if you want to overload Assignment operator
        public HelloWorldEzAPIExampleWithDimodelo(Package p) : base(p) { }

        public HelloWorldEzAPIExampleWithDimodelo() : base() { }

        // All the tasks, components and connection managers which should automatically be
        // assigned when SSIS package is assigned to EzPackage MUST BE PUBLIC MEMBERS.

        public EzExecSqlTask ExecSQL; //Execute SQL Command task
        
        
  
public  string Execute(DimodeloMetaData metaData, string generateForUid)
{

    // Get the Dimodelo Meta Data as an XElement. There are other options. 
    //The Meta Data can be retrieved as an XML string, or an XPathNavigator also. 
    XElement DimodeloMetaDataXML = metaData.getXElement();
    
    //Extract out of DimodeloMetaDataXML, using XLINQ syntax, the XML for the Fact that the 
    //ezAPI template is generating For (identified by the unique identifier passed in the
    // generateForUid parameter. All other meta data comes from a child element of this.
 
    IEnumerable<XElement> resultset =
    from el in DimodeloMetaDataXML.Element("Facts").Elements("Fct")
    where el.Element("Uid").Value == generateForUid
    select el;
            
    // there will only be one result in the result set.
    XElement factXML = null;
    foreach (XElement el in resultset)
        factXML = el;

    // Add the ExecSQL Command task to the package 
    ExecSQL = new EzExecSqlTask(this);
            
    // Assign a name to the ExecSQL command from Dimodelo Meta Data
    ExecSQL.Name = "Select " + factXML.Element("Name").Value;
                        
    //Return the resulting package back to the caller as a string
    return this.SaveToXML();
            
}

        static void Main(string[] args)
        {

            // This method emulates how Dimodelo Architect calls the Execute method of 
            // your EzAPI package generation class. Each Dimodelo Architect project 
            // contains a file called Dimodelo_Meta_Data.xml which contains your
            // Data Warehouse design data in XML format. To access the design data use 
            // the following code, with the path to the relevant Dimodelo_Meta_Data.xml.

            DimodeloMetaData dmd = new DimodeloMetaData("Dimodelo_Meta_Data.xml");

            // Your ezAPI template will operate on only one Fact or Dimension at a time.
            // This is passed to the Execute method using the generateFor parameter.
            // The value of the generateFor parameter is the Uid of the Fact/Dimension in Dimodelo_Meta_Data.xml. 
            // The GUID value below is a known Uid of the a Fact table in Dimodelo_Meta_Data.xml. 

            String generateFor = "4c47c33a-562c-4d5c-9bf1-ef739cbf7ac1";

            // Execute the Hello World with Dimodelo Example

            HelloWorldEzAPIExampleWithDimodelo pkgGenerator = new HelloWorldEzAPIExampleWithDimodelo();

            string HelloWorldPkg = pkgGenerator.Execute(dmd, generateFor);

            System.IO.StreamWriter file = new System.IO.StreamWriter("HelloWorldDimodelo.dtsx");
            file.WriteLine(HelloWorldPkg);
            file.Close();

            System.Console.WriteLine("Hello World With Dimodelo Generation Complete");

        }

    }


}
